class BaseFrontMatterProcessor(object):

    def __init__(self, **params):
        """Set processor-specific parameters.
        """
        pass

    def process(self, posts, allow_overwrite):
        """Process a list of posts and update their front matters.
        """
        pass



from .base import BaseFrontMatterProcessor
from .lastmod import LastModifiedDateExtractor
from .opengraph import OpenGraphMediaExtractor
from .recommender import Recommender


__all__ = ['BaseFrontMatterProcessor',
           'LastModifiedDateExtractor',
           'OpenGraphMediaExtractor',
           'Recommender']



from .post import Post

import os
from pathlib import Path

class StaticSitePostsHandler(object):

    def __init__(self, path_dir, ignore_files=None, encoding='utf-8'):
        assert os.path.isdir(os.path.expanduser(path_dir)), \
            f'path {path_dir} is not a directory or does not exist'

        exts = ['.md', '.html']
        self.paths = [p for p in Path(path_dir).rglob('*') if p.suffix in exts]
        self.processors = []
        self.encoding = encoding
        self.ignore_files = [] if ignore_files is None else ignore_files

    def register_processor(self, processor):
        """Add a front matter processor to the queue.
        """
        self.processors.append(processor)

    def execute(self):
        """Load all posts under the document root, process front matters, and
        write the results onto the original files.
        """
        posts = self.load_posts()
        for processor in self.processors:
            processor.process(posts)
        self.save_posts(posts)

    def load_posts(self):
        """Get a list of posts under the document root, excluding invalid posts
        e.g., draft articles, empty files.
        """
        posts = []
        for path in self.paths:
            if path.name in self.ignore_files:
                continue

            post = Post.load(path, self.encoding)
            if post.is_valid():
                posts.append(post)
        return posts

    @staticmethod
    def save_posts(posts):
        """Call a save operation for given posts.
        """
        for post in posts:
            post.save()



import re
import yaml

RE_FRONT_MATTER = re.compile(r'---\n([\s\S]*?\n)---\n')
RE_FILTERS = [
    # HTML tag
    re.compile(r'<.*?>'),
    # Markdown codefence / math block
    re.compile(r'^(\$\$|```)(.*)\n(.*\n)+(\$\$|```)', re.MULTILINE),
    # inline math
    re.compile(r'\$(.*)\$'),
    # URL
    re.compile(r'https?:\/\/[\S]+')
]


class Post(object):

    def __init__(self, path, front_matter, raw_content, content,
                 encoding='utf-8'):
        self.path = path
        self.front_matter = front_matter
        self.raw_content = raw_content
        self.content = content
        self.encoding = encoding

    def update(self, key, value, allow_overwrite=False):
        """Update value of a specific front matter element.

        Parameters
        ----------
        key : str
            Front matter key to update.

        value : str or list
            Updated value of the target key.

        allow_overwrite : bool, default=False
            What to do if the target key already exisits. Its value is
            overwritten when ``allow_overwrite=True``.
        """
        if key in self.front_matter and not allow_overwrite:
            return
        if value is None or len(value) == 0:
            return
        self.front_matter[key] = value

    def update_all(self, target, allow_overwrite=False):
        """Update multiple keys in the front matter all at once.

        Parameters
        ----------
        target : dict
            Updated front matter key-value pairs.

        allow_overwrite : bool, default=False
            If True, all keys in ``target`` are first cleared from the current
            front matter, then their values are freshly inserted. Otherwise,
            delegate every single key-value pair to ``update`` one-by-one.
        """
        if allow_overwrite:
            self.__remove_all(target.keys())

        for key, value in target.items():
            self.update(key, value, allow_overwrite)

    def is_draft(self):
        # avoid processing draft articles
        return ('draft' in self.front_matter and self.front_matter['draft']) \
                or ('published' in self.front_matter and
                    not self.front_matter['published'])

    def is_valid(self):
        return self.front_matter is not None \
                and len(self.content) > 0 \
                and not self.is_draft()

    def save(self):
        if not self.is_valid():
            return

        m = RE_FRONT_MATTER.search(self.raw_content)
        # Don't expand array value to markdown-style list notation
        # https://github.com/yaml/pyyaml/pull/256
        value_types = {type(value) for value in self.front_matter.values()}
        flow_style = None if list in value_types else False
        with open(self.path, 'w', encoding=self.encoding) as f:
            content = self.raw_content.replace(
                m.group(1),
                yaml.dump(self.front_matter, allow_unicode=True,
                          default_flow_style=flow_style,
                          sort_keys=False)
            )
            f.write(content)

    @staticmethod
    def load(path, encoding="utf-8"):
        with open(path, encoding=encoding) as f:
            raw_content = f.read()

        front_matter = None
        content = raw_content

        m = RE_FRONT_MATTER.search(raw_content)

        if m is not None:
            front_matter = yaml.safe_load(m.group(1))

            # remove front matter
            content = raw_content.replace(m.group(0), '')

        for re_filter in RE_FILTERS:
            content = re_filter.sub('', content).strip()

        return Post(path, front_matter, raw_content, content, encoding)

    def __remove(self, key):
        """Delete a specific front matter element.

        Parameters
        ----------
        key : str
            Front matter key to delete.
        """
        if key in self.front_matter:
            del self.front_matter[key]

    def __remove_all(self, keys):
        """Delete multiple front matter elements.

        Parameters
        ----------
        keys : list
            Front matter keys to delete.
        """
        for key in keys:
            self.__remove(key)



from .base import BaseFrontMatterProcessor

import re


class OpenGraphMediaExtractor(BaseFrontMatterProcessor):

    """Extract image, audio, and/or video paths from an article and add them to
    dedicated front matter elements as lists of permalinks.

    In particular, the processor modifies three front matter elements with
    keys: ``images``, ``audio``, and ``videos``.

    Parameters
    ----------
    image_base : str, default=None
        Non-file name part of image permalink. If an article refers to an image
        as ``/images/foo.png``, ``image_base`` is expected to be ``'/images'``.
        The extractor looks for all permalinks that match
        ``/images/*.{jpg,jpeg,png}``. Won't do anything if ``None``.

    audio_base: str, default=None
        Non-file name part of audio permalink. If an article refers to an audio
        as ``/audio/foo.mp3``, ``audio_base`` is expected to be ``'/audio'``.
        The extractor looks for all permalinks that match
        ``/audio/*.{mp3,wav}``. Won't do anything if ``None``.

    video_base: str, default=None
        Non-file name part of video permalink. If an article refers to a video
        as ``/movies/foo.mp4``, ``video_base`` is expected to be ``'/movies'``.
        The extractor looks for all permalinks that match
        ``/movies/*.{mp4,wov}``. Won't do anything if ``None``.
    """

    def __init__(self, image_base=None, audio_base=None, video_base=None):
        self.pattern = dict()
        if image_base is not None:
            self.pattern['images'] = re.compile(
                rf'{re.escape(image_base)}/.+?(?:\.jpg|\.jpeg|\.png)'
            )

        if audio_base is not None:
            self.pattern['audio'] = re.compile(
                rf'{re.escape(audio_base)}/.+?(?:\.mp3|\.wav)'
            )

        if video_base is not None:
            self.pattern['videos'] = re.compile(
                rf'{re.escape(video_base)}/.+?(?:\.mp4|\.mov)'
            )

    def process(self, posts, allow_overwrite=False):
        for post in posts:
            target = dict()
            for key, re_path in self.pattern.items():
                re_path = self.pattern[key]
                paths = re_path.findall(post.content)
                target[key] = paths
            post.update_all(target, allow_overwrite)



from .base import BaseFrontMatterProcessor

import os
from urllib.parse import urljoin
import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity


class Recommender(BaseFrontMatterProcessor):

    """Tokenize contents, extract keywords, and generate a list of recommended
    article paths.


    Parameters
    ----------
    permalink_base : str, default=''
        Non-file name part of your article permalink. For example, if an
        article saved under ``/path/to/articles/artcile-aaa.md`` eventually
        becomes accessible from a URL
        ``https://awesome-website.com/diary/post/article-aaa/``, it means
        ``permalink_base='/diary/post'``; the permalink is a concatination of
        the base and file name ``article-aaa``, excluding the extension.

    topk : int, default=3
        Number of recommended articles.

    lower_path : boolean, default=True
        Make paths in recommendations field lower case

    **kwargs : dict
        Keyword arguments to be used to initialize sklearn's
        ``TfidfVectorizer``.

    Notes
    -----
    One possibility of using ``**kwargs`` is to explicitly provide
    ``stop_words`` so the processor can filter out meaningless terms (e.g.,
    'is', 'this', 'of'). Besides sklearn's ``stop_words='english'``, you could
    leverage an arbitrary set of words; if you have installed ``spacy`` in your
    environment, for example, their stop word list will give wider coverage of
    words.

    Examples
    --------
    >>> from prelims import Post
    >>> from prelims.processor import Recommender
    >>> from spacy.lang import en
    >>> post_a = Post('/path/to/posts/a.md', {'title': 'foo'},
    ...               '---\ntitle: foo\n---\n\nHellow world.',
    ...               'Hello world.')
    >>> post_b = Post('/path/to/posts/b.md', {'title': 'bar'},
    ...               '---\ntitle: bar\n---\n\nThis is a pen.',
    ...               'This is a pen.')
    >>> recommender = Recommender(permalink_base='/posts',
    ...                           stop_words=en.STOP_WORDS)
    >>> recommender.process([post_a, post_b])
    >>> post_a.front_matter.keywords
    ['world', 'hello', 'pen']
    >>> post_a.front_matter.recommendations
    ['/posts/b/']
    >>> post_b.front_matter.keywords
    ['pen', 'world', 'hello']
    >>> post_b.front_matter.recommendations
    ['/posts/a/']
    """

    def __init__(self, permalink_base='', topk=3, lower_path=True, **kwargs):
        self.permalink_base = permalink_base
        self.topk = topk
        self.lower_path = lower_path

        vectorizer_kwargs = TfidfVectorizer.__init__.__kwdefaults__
        for arg, value in kwargs.items():
            if arg in vectorizer_kwargs:
                vectorizer_kwargs[arg] = value
        self.vectorizer_kwargs = vectorizer_kwargs

    def process(self, posts, allow_overwrite=True):
        """Extract keywords and generate a list of recommended articles
        based on the content-based filtering technique.
        """
        contents = [post.content for post in posts]
        paths = [post.path for post in posts]

        # build model
        vectorizer = TfidfVectorizer(**self.vectorizer_kwargs)

        tfidf = vectorizer.fit_transform(contents)

        indices = tfidf.toarray().argsort(axis=1, kind='stable')[:, ::-1]
        keywords = np.array(vectorizer.get_feature_names_out())[indices]

        similarities = cosine_similarity(tfidf)

        for i in range(len(contents)):
            # find top-k most-similar articles
            # (except for target article itself which is similarity=1.0)
            top_indices = np.argsort(
                similarities[i, :], kind='stable')[::-1][1:(self.topk + 1)]
            recommend_permalinks = [
                self.__path_to_permalink(paths[j]) for j in top_indices
            ]

            posts[i].update_all({
                'keywords': keywords[i, :10].tolist(),
                'recommendations': recommend_permalinks
            }, allow_overwrite)

    def __path_to_permalink(self, path):
        """Convert a file path into a permalink, which is a part of final URL
        excluding a file extension.
        """
        file = path.stem
        if file == 'index':
            file = path.parent.name
        if self.lower_path:
            file = file.lower()
        return urljoin(f'{self.permalink_base}/', f'{file}/')



from setuptools import setup, find_packages

DISTNAME = 'prelims'
VERSION = '0.0.7'
DESCRIPTION = 'Front matter post-processor for static site generators'
with open('README.md') as f:
    LONG_DESCRIPTION = f.read()
LONG_DESCRIPTION_CONTENT_TYPE = 'text/markdown'
AUTHOR = 'Takuya Kitazawa'
AUTHOR_EMAIL = 'k.takuti@gmail.com'
LICENSE = 'MIT'
URL = 'https://github.com/takuti/prelims'
DOWNLOAD_URL = 'https://pypi.org/project/prelims/#files'


def setup_package():
    metadata = dict(
        name=DISTNAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        license=LICENSE,
        url=URL,
        download_url=DOWNLOAD_URL,
        classifiers=[
            'Intended Audience :: Developers',
            'License :: OSI Approved',
            'Programming Language :: Python',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
            'Topic :: Office/Business :: News/Diary',
            'Topic :: Software Development',
            'Topic :: Text Processing :: Markup',
            'Development Status :: 3 - Alpha',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX',
            'Operating System :: Unix',
            'Operating System :: MacOS',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9'
        ],
        packages=find_packages(exclude=['*tests*']),
        python_requires='>=3.7',  # follow sklearn
        install_requires=['numpy>=1.14.6', 'scikit_learn>=1.0', 'PyYAML>=5.1'],
    )

    setup(**metadata)


if __name__ == '__main__':
    setup_package()



from .base import BaseFrontMatterProcessor

from datetime import datetime
import os
import platform
import subprocess


class LastModifiedDateExtractor(BaseFrontMatterProcessor):

    """Get last modified date based on git last commit date, or file metadata
    if the file is not committed.
    """

    def __init__(self):
        pass

    def process(self, posts, allow_overwrite=True):
        for post in posts:
            try:
                cmd = "git log -1 --format='%ad' --date=format:'%Y-%m-%d' " + \
                        str(post.path)
                date = self.__read_stdout(cmd)
            except RuntimeError:
                date = self.__get_file_mtime(post.path)
            else:
                if len(date) == 0:
                    date = self.__get_file_mtime(post.path)
            post.update("lastmod", date, allow_overwrite)

    def __read_stdout(self, cmd):
        """Run a shell command and get STDOUT.
        """
        try:
            return subprocess.check_output(cmd, shell=True,
                                           encoding="utf-8").rstrip()
        except subprocess.CalledProcessError as e:
            raise RuntimeError(e)

    def __get_file_mtime(self, path):
        """https://stackoverflow.com/questions/237079/
        """
        if platform.system() == 'Windows':
            ts = os.path.getmtime(path)
        else:
            ts = os.stat(path).st_mtime
        return datetime.fromtimestamp(ts).strftime("%Y-%m-%d")
